'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

exports.default = sanitizeAmpHtml;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
function sanitizeAmpHtml(html) {
  var styleId = 0;
  var styles = [];
  var _global = global,
      fns = _global.fns;

  var dom = fns.init$(html);
  var _global2 = global,
      $ = _global2.$; // note this must be after fns.init$(), which ensures that $ is exported globally

  // Provide support for amp bind expressions on custom amp tags
  // React removes attributes like [selected]="selectedTab" and throws a warning, but allows attributes without brackets,
  // So we use amp-bind="selected:selectedTab" instead.
  // Example: <amp-selector amp-bind="selected:selectedTab,foo:bar"> => <amp-selector [selected]="selectedTab" [foo]="bar">

  dom.$('*[amp-bind]').each(function (i, el) {
    var $el = $(el);
    var expressions = $el.attr("amp-bind").split(/,\s*/);

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = expressions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var expression = _step.value;

        var _expression$split = expression.split('=>'),
            _expression$split2 = (0, _slicedToArray3.default)(_expression$split, 2),
            name = _expression$split2[0],
            value = _expression$split2[1];

        $el.attr('[' + name + ']', value);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    $el.removeAttr('amp-bind');
  });

  // replace attr="true" with attr
  dom.$('*').each(function (i, el) {
    var $el = $(el);
    for (var name in el.attribs) {
      var value = el.attribs[name];
      if (value === "true") {
        $el.attr(name, '');
      }
    }
  });

  // consolidate all style tags in the header into one
  dom.$('style:not([amp-boilerplate])').each(function (i, style) {
    var $style = $(style);
    styles.push($style.html().replace(/\s*!important\s*/g, '')); // amp doesn't allow !important
    $style.remove();
  });

  dom.$('img').each(function (i, img) {
    var $img = $(img);
    var $ampImg = $('<amp-img layout="intrinsic"></amp-img>');

    if ($img.attr('src')) $ampImg.attr('src', $img.attr('src'));
    if ($img.attr('alt')) $ampImg.attr('alt', $img.attr('alt'));
    if ($img.attr('height')) $ampImg.attr('height', $img.attr('height'));
    if ($img.attr('width')) $ampImg.attr('width', $img.attr('width'));
    if ($img.attr('data-height')) $ampImg.attr('height', $img.attr('data-height'));
    if ($img.attr('data-width')) $ampImg.attr('width', $img.attr('data-width'));

    for (var name in img.attribs) {
      if (name.startsWith('data-amp-')) {
        $ampImg.attr(name.replace(/^data-amp-/, ''), img.attribs[name]);
      }
    }

    $img.replaceWith($ampImg);
  });

  // remove focusable attribute on all svg elements
  dom.$('svg[focusable]').removeAttr('focusable');
  dom.$('svg[xlink]').removeAttr('xlink');

  // material-ui puts this on tab underlines
  dom.$('div[direction]').removeAttr('direction');

  // move amp-sidebar into direct child of body
  var sidebar = dom.$('amp-sidebar').get();
  if (sidebar) {
    $(sidebar).remove();
    dom.$('body').append(sidebar);
  }

  // move all inline styles to classes in the main style tag
  dom.$('*[style]').each(function (i, el) {
    var $el = $(el);
    var className = 'mi' + styleId++;
    styles.push('.' + className + ' {' + $el.attr('style') + '}');
    $el.removeAttr('style');
    $el.addClass(className);
  });

  // write the single style tag
  dom.$('head').append('<style amp-custom>' + styles.join('\n') + '</style>');

  return dom.$.html();
}
//# sourceMappingURL=sanitizeAmpHtml.js.map