"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mobxStateTree = require("mobx-state-tree");

var _ProductModelBase = require("./ProductModelBase");

var _ProductModelBase2 = _interopRequireDefault(_ProductModelBase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A base model for the shopping cart that automatically fires added_to_cart and removed_from_cart analytics events
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var CartModelBase = _mobxStateTree.types.model("CartModelBase", {
  items: _mobxStateTree.types.optional(_mobxStateTree.types.array(_ProductModelBase2.default), [])
}).views(function (self) {
  return {
    get quantity() {
      var total = 0;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = self.items[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var item = _step.value;

          total += item.quantity;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return total;
    }
  };
}).actions(function (self) {
  return {
    /**
     * Adds a product to the cart
     * @param {ProductModelBase} product 
     */
    add: function add(product) {
      self.items.push((0, _mobxStateTree.clone)(product));
    },

    /**
     * Removes a product from the cart
     * @param {ProductModelBase} product 
     */
    remove: function remove(product) {
      (0, _mobxStateTree.detach)(product);
    }
  };
});

exports.default = CartModelBase;
//# sourceMappingURL=CartModelBase.js.map