"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _mobxStateTree = require("mobx-state-tree");

var _SelectionModelBase = require("./SelectionModelBase");

var _SelectionModelBase2 = _interopRequireDefault(_SelectionModelBase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var ProductModelBase = _mobxStateTree.types.model('ProductModelBase', {
  id: _mobxStateTree.types.identifier(_mobxStateTree.types.string),
  url: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  name: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  description: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  size: _mobxStateTree.types.maybe(_SelectionModelBase2.default),
  color: _mobxStateTree.types.maybe(_SelectionModelBase2.default),
  quantity: _mobxStateTree.types.optional(_mobxStateTree.types.number, 1),
  rating: _mobxStateTree.types.maybe(_mobxStateTree.types.number),
  reviewCount: _mobxStateTree.types.maybe(_mobxStateTree.types.number),
  basePrice: _mobxStateTree.types.maybe(_mobxStateTree.types.number),
  brand: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  images: _mobxStateTree.types.optional(_mobxStateTree.types.array(_mobxStateTree.types.string), []),
  thumbnails: _mobxStateTree.types.optional(_mobxStateTree.types.array(_mobxStateTree.types.string), []),
  thumbnail: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  loadingImages: false
}).views(function (self) {
  return {
    shouldApplyPatchOnPop: function shouldApplyPatchOnPop(patch) {
      return patch.page === 'Product';
    },

    get price() {
      var size = self.size && self.size.selected;

      if (size && size.price) {
        return size.price;
      } else {
        return self.basePrice;
      }
    }
  };
}).actions(function (self) {
  return {
    afterCreate: function afterCreate() {
      (0, _mobxStateTree.onPatch)(self, function (patch) {
        if (patch.path === '/color/selected') {
          // fetch images when a new color is selected
          self.fetchImages();
        }
      });
    },


    /**
     * Updates the quantity and fires an analytics event
     * @param {Number} quantity 
     */
    setQuantity: function setQuantity(quantity) {
      self.quantity = Math.max(1, quantity);
    },

    /**
     * Get images for the selected color
     */
    fetchImages: function fetchImages() {
      var _window$location = window.location,
          pathname = _window$location.pathname,
          search = _window$location.search;

      var selected = self.color.selected;
      self.loadingImages = true;

      if (selected) {
        fetch(pathname + "/images/" + selected.id + ".json" + search).then(function (res) {
          return res.json();
        }).then(function (state) {
          return self.apply((0, _extends3.default)({}, state, { loadingImages: false }));
        }).catch(function () {
          return self.apply({ loadingImages: false });
        });
      }
    },

    /**
     * Update the images based on the response from the images handler
     * @param {String[]} images 
     */
    apply: function apply(state) {
      Object.assign(self, state);
    }
  };
});

exports.default = ProductModelBase;
//# sourceMappingURL=ProductModelBase.js.map