'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ResultsGroupModel = exports.ResultsModel = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fetch = require('fetch');

var _fetch2 = _interopRequireDefault(_fetch);

var _lodash = require('lodash');

var _mobxStateTree = require('mobx-state-tree');

var _fetchLatest = require('../fetchLatest');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Ensures that responses are returned in order and that the previous request is canceled when a new request is sent.
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var fetchSearchResults = (0, _fetchLatest.fetchLatest)(_fetch2.default);

/**
 * Represents an individual link within a group
 */
var ResultsModel = exports.ResultsModel = _mobxStateTree.types.model('ResultsModel', {
  /**
   * The link text
   */
  text: _mobxStateTree.types.string,
  /**
   * The link's target url
   */
  url: _mobxStateTree.types.string,
  /**
   * A url for a thumbnail image
   */
  thumbnail: _mobxStateTree.types.maybe(_mobxStateTree.types.string),
  /**
   * The thumbnail height
   */
  thumbnailHeight: 120,
  /**
   * The thumbnail width
   */
  thumbnailWidth: 120
});

/**
 * Represents a group of links withing the search results
 */
var ResultsGroupModel = exports.ResultsGroupModel = _mobxStateTree.types.model('ResultsGroupModel', {
  /**
   * The caption to display at the top of the group
   */
  caption: _mobxStateTree.types.string,
  /**
   * The list of links to display
   */
  results: _mobxStateTree.types.optional(_mobxStateTree.types.array(ResultsModel), [])
}).views(function (self) {
  return {
    get thumbnails() {
      return self.results.some(function (r) {
        return r.thumbnail != null;
      });
    }
  };
});

/**
 * The base model for site-wide searches.
 */
var SearchModelBase = _mobxStateTree.types.model("SearchModelBase", {
  /**
   * The search phrase entered by the user
   */
  text: '',
  /**
   * The resulting groups
   */
  groups: _mobxStateTree.types.optional(_mobxStateTree.types.array(ResultsGroupModel), []),
  /**
   * True to show the loading sprinner
   */
  loading: false,
  /**
   * True to show the search popup
   */
  show: false
}).actions(function (self) {
  return {
    /**
     * Set `true` to show the search popup, `false` to hide.
     * @param {Boolean} show 
     */
    toggle: function toggle(show) {
      self.show = show;
    },

    /**
     * Update the search text
     * @param {String} text 
     */
    setText: function setText(text) {
      self.text = text;
      self.loading = self.text.trim().length > 0;
      self.submit(text);
    },

    /**
     * Set `true` to show the loading spinner, `false` to hide.
     * @param {Boolean} state 
     */
    setLoading: function setLoading(loading) {
      self.loading = loading;
    },

    /**
     * Submit the search for the given keyword.  Fetched calls are automatically debounced and serialized so that
     * results are received in order.
     * @param {String} keyword
     */
    submit: (0, _lodash.debounce)(function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(keyword) {
        var state;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (keyword) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return');

              case 2:
                _context.prev = 2;
                _context.next = 5;
                return fetchSearchResults('/search/suggest.json?q=' + encodeURIComponent(keyword)).then(function (res) {
                  return res.json();
                });

              case 5:
                state = _context.sent;

                self.setGroups(state.search.groups);
                self.setLoading(false);
                _context.next = 15;
                break;

              case 10:
                _context.prev = 10;
                _context.t0 = _context['catch'](2);

                if (_fetchLatest.StaleResponseError.is(_context.t0)) {
                  _context.next = 15;
                  break;
                }

                self.setLoading(false);
                throw _context.t0;

              case 15:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined, [[2, 10]]);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }(), 500),
    /**
     * Set the groups to be displayed
     * @param {ResultsGroupModel[]} groups 
     */
    setGroups: function setGroups(groups) {
      self.groups = groups;
    }
  };
});

exports.default = SearchModelBase;
//# sourceMappingURL=SearchModelBase.js.map