'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = persist;

var _mobxStateTree = require('mobx-state-tree');

var _store = require('store2');

var _store2 = _interopRequireDefault(_store);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *  import { persist } from 'path/to/persist'
 * 
 *  export const ModelName = types
 *   .model('ModelName', {
 *     keyOne: 'includeMe',
 *     keyTwo: 'excludeMe'
 *   })
 *   .actions(self => ({
 *     afterCreate() {
 *       persist('storageName', self, {
 *         keyOne: true,
 *       })
 *     }
 *   }))
 * 
 * @param {String} name The name of the item in storage
 * @param {Model} store The mobx state tree model to store
 * @param {Object} schema OPTIONAL object of model keys, used to limit what gets stored - all keys stored by default
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
function persist(name, store) {
  var schema = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};


  (0, _mobxStateTree.onSnapshot)(store, function (_snapshot) {
    var snapshot = (0, _extends3.default)({}, _snapshot);
    // use only keys found in schema
    // if schema is empty store everything
    if (Object.keys(schema).length) {
      Object.keys(snapshot).forEach(function (key) {
        if (!schema[key]) {
          delete snapshot[key];
        }
      });
    }
    _store2.default.set(name, snapshot);
  });

  var snapshot = _store2.default.get(name);

  if (snapshot) {
    (0, _mobxStateTree.applySnapshot)(store, snapshot);
  }
};
//# sourceMappingURL=persist.js.map