'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _asyncGenerator2 = require('babel-runtime/helpers/asyncGenerator');

var _asyncGenerator3 = _interopRequireDefault(_asyncGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncIterator2 = require('babel-runtime/helpers/asyncIterator');

var _asyncIterator3 = _interopRequireDefault(_asyncIterator2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _routeParser = require('route-parser');

var _routeParser2 = _interopRequireDefault(_routeParser);

var _lodash = require('lodash.isfunction');

var _lodash2 = _interopRequireDefault(_lodash);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _lodash3 = require('lodash');

var _serviceWorker = require('./serviceWorker');

var _parseMultipartRequest = require('./parseMultipartRequest');

var _parseMultipartRequest2 = _interopRequireDefault(_parseMultipartRequest);

var _Response = require('./Response');

var _Response2 = _interopRequireDefault(_Response);

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provides routing for MUR-based applications and PWAs.  This class is innspired by express and uses https://github.com/rcs/route-parser,
 * which supports sophisticated pattern matching including optional paths, params, and splatting.
 *
 * Example:
 *
 *  const router = new Router()
 *
 *  router.get('/products/:id', ({ id }) => {
 *    // fetch product from upstream API (you'll need to write this function)
 *    return fetchProduct(id).then(result => {
 *      return result.product // this will be the result of router.run()
 *    })
 *  })
 *
 *  // assuming env.path = /products/1 and env.method = 'GET'
 *  router.run() // => the details for product 1
 *
 * Routes can be divided into multiple files to increase maintainability using the "use()" method.  For example:
 *
 *  // /scripts/api/router.js
 *
 *  const appShell = require('/build/index.html.js)
 *
 *  module.exports = new Router()
 *    .fallback(() => appShell) // render the PWA's app shell for all unmatched routes
 *    .use('/products', require('/api/products.js'))
 *
 *
 *  // /scripts/api/products.js
 *
 *  module.exports = new Router()
 *    .get('/:id', ({ id }) => new Promise((resolve, reject) => {
 *      // fetch product from upstream API...
 *     }))
 *
 *
 *  // /scripts/index.js
 *
 *  const router = require('/api/router')
 *
 *  module.exports = function() {
 *    // ...
 *    router.run().then((result) => {
 *      const body = typeof result === 'string' ? result : JSON.stringify(result)
 *      sendResponse({ body, htmlparsed: true })
 *    })
 *  }
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var Router = function (_EventEmitter) {
  (0, _inherits3.default)(Router, _EventEmitter);

  function Router() {
    var _this2 = this;

    (0, _classCallCheck3.default)(this, Router);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Router.__proto__ || Object.getPrototypeOf(Router)).call(this));

    _this.onLocationChange = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(callback, location, action) {
        var pathname, search, request, response, state, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, _state;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(action === 'REPLACE')) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return');

              case 2:
                if (!(location.pathname === _this.prevLocation.pathname && location.search === _this.prevLocation.search)) {
                  _context.next = 4;
                  break;
                }

                return _context.abrupt('return');

              case 4:

                _this.prevLocation = location; // this needs to come before handlers are called or going back while async handlers are running will lead to a broken state

                pathname = location.pathname, search = location.search;
                request = { pathname: pathname, search: search, method: 'GET' };
                response = new _Response2.default(request);
                state = location.state;


                if (state) {
                  callback(state, action); // called when restoring history state and applying state from Link components
                }

                _this.emit('before', { request: request, response: response });

                if (!(action === 'PUSH' || !state)) {
                  _context.next = 48;
                  break;
                }

                /*
                 * Why limit action to PUSH here? POP indicates that the user is going back or forward
                 * In those cases, if we have location.state, we can assume it's the full state.  We don't need to
                 * do anything for replace.
                 */
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 15;
                _iterator = (0, _asyncIterator3.default)(_this.run(request, response, { historyState: state }));

              case 17:
                _context.next = 19;
                return _iterator.next();

              case 19:
                _step = _context.sent;
                _iteratorNormalCompletion = _step.done;
                _context.next = 23;
                return _step.value;

              case 23:
                _value = _context.sent;

                if (_iteratorNormalCompletion) {
                  _context.next = 30;
                  break;
                }

                _state = _value;

                callback(_state, action);

              case 27:
                _iteratorNormalCompletion = true;
                _context.next = 17;
                break;

              case 30:
                _context.next = 36;
                break;

              case 32:
                _context.prev = 32;
                _context.t0 = _context['catch'](15);
                _didIteratorError = true;
                _iteratorError = _context.t0;

              case 36:
                _context.prev = 36;
                _context.prev = 37;

                if (!(!_iteratorNormalCompletion && _iterator.return)) {
                  _context.next = 41;
                  break;
                }

                _context.next = 41;
                return _iterator.return();

              case 41:
                _context.prev = 41;

                if (!_didIteratorError) {
                  _context.next = 44;
                  break;
                }

                throw _iteratorError;

              case 44:
                return _context.finish(41);

              case 45:
                return _context.finish(36);

              case 46:
                _context.next = 49;
                break;

              case 48:
                if (state) {
                  callback(state, action); // called when restoring history state and applying state from Link components
                }

              case 49:
                _this.emit('after', { request: request, response: response });

              case 50:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this2, [[15, 32, 36, 46], [37,, 41, 45]]);
      }));

      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }();

    _this.routes = [];
    _this.isBrowser = process.env.MOOV_RUNTIME === 'client';

    _this.fallbackHandlers = [{
      runOn: { client: true, server: true },
      fn: function fn() {
        return { page: '404' };
      }
    }];

    _this.errorHandler = function (e, params, request, response) {
      console.error('Error caught with params', params, ' and with message:', e.message);

      if (!_this.isBrowser) {
        response.status(500, 'error');
      }

      return { page: 'Error', error: e.message, stack: e.stack, loading: false };
    };
    return _this;
  }

  (0, _createClass3.default)(Router, [{
    key: 'pushRoute',
    value: function pushRoute(method, path, handlers) {
      this.routes.push({ path: new _routeParser2.default(path + '.:format'), method: method, handlers: handlers });
      this.routes.push({ path: new _routeParser2.default(path), method: method, handlers: handlers });
      return this;
    }

    /**
     * Registers a GET route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'get',
    value: function get(path) {
      for (var _len = arguments.length, handlers = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        handlers[_key - 1] = arguments[_key];
      }

      return this.pushRoute('GET', path, handlers);
    }

    /**
     * Registers a POST route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'post',
    value: function post(path) {
      for (var _len2 = arguments.length, handlers = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        handlers[_key2 - 1] = arguments[_key2];
      }

      return this.pushRoute('POST', path, handlers);
    }

    /**
     * Registers a PATCH route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'patch',
    value: function patch(path) {
      for (var _len3 = arguments.length, handlers = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        handlers[_key3 - 1] = arguments[_key3];
      }

      return this.pushRoute('PATCH', path, handlers);
    }

    /**
     * Registers a PUT route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'put',
    value: function put(path) {
      for (var _len4 = arguments.length, handlers = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
        handlers[_key4 - 1] = arguments[_key4];
      }

      return this.pushRoute('PUT', path, handlers);
    }

    /**
     * Registers a DELETE route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'delete',
    value: function _delete(path) {
      for (var _len5 = arguments.length, handlers = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
        handlers[_key5 - 1] = arguments[_key5];
      }

      return this.pushRoute('DELETE', path, handlers);
    }

    /**
     * Registers an OPTIONS route
     * @param {String} path A path pattern
     * @param {Function} callback A function that is passed the params as an object and returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'options',
    value: function options(path) {
      for (var _len6 = arguments.length, handlers = Array(_len6 > 1 ? _len6 - 1 : 0), _key6 = 1; _key6 < _len6; _key6++) {
        handlers[_key6 - 1] = arguments[_key6];
      }

      return this.pushRoute('OPTIONS', path, handlers);
    }

    /**
     * Designates the handlers for unmatched routes
     * @param {Function} callback A function that returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'fallback',
    value: function fallback() {
      for (var _len7 = arguments.length, handlers = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        handlers[_key7] = arguments[_key7];
      }

      this.fallbackHandlers = handlers;
      return this;
    }

    /**
     * Sets the handler for errors thrown during route handling
     * @param {Function} handler A function that returns a promise that resolves to the content to return
     * @return {Router} this
     */

  }, {
    key: 'error',
    value: function error(handler) {
      this.errorHandler = handler;
      return this;
    }

    /**
     * Registers a set of nested routes.
     *
     * Example:
     *
     *  Router root = new Router()
     *
     *  Router products = new Router()
     *  products.get('/:id', ({ id }) => {
     *    return Promise.resolve(id)
     *  })
     *
     *  root.use('/products', products)
     *
     *  // url: /products/1
     *  root.run().then(result => console.log(result)) // => 1
     *
     * @param {String} path The parent path
     * @param {Router} router A router to handle the nested routes
     * @return {Router} this
     */

  }, {
    key: 'use',
    value: function use(path, router) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = router.routes[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var route = _step2.value;
          var routePath = route.path,
              rest = (0, _objectWithoutProperties3.default)(route, ['path']);

          this.routes.push((0, _extends3.default)({ path: new _routeParser2.default(path + routePath.spec) }, rest));
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return this;
    }

    /**
     * @private
     * Caches the initialState (json) and HTML using the service worker.
     * @param {Object} request
     */

  }, {
    key: 'cacheInitialState',
    value: function cacheInitialState(request) {
      var pathname = request.pathname,
          search = request.search;

      (0, _serviceWorker.cache)(pathname + search, '<!DOCTYPE html>\n' + document.documentElement.outerHTML);
    }

    /**
     * Configures service worker runtime caching options
     * @param {Object} options
     * @param {Object} options.cacheName The name of the runtime cache
     * @param {Object} options.maxEntries The max number of entries to store in the cache
     * @param {Object} options.maxAgeSeconds The TTL in seconds for entries
     * @return {Router} this
     */

  }, {
    key: 'configureClientCache',
    value: function configureClientCache(options) {
      (0, _serviceWorker.configureCache)(options);
      return this;
    }

    /**
     * Gets the server cache key for the matching route.
     * @param {Object} request
     * @param {Object} defaults The default values used for the cache key
     * @return {Object} An object populate with keys and values that when hashed, make up the cache key
     */

  }, {
    key: 'getCacheKey',
    value: function getCacheKey(request, defaults) {
      var _findMatchingRoute = this.findMatchingRoute(request),
          match = _findMatchingRoute.match;

      var handlers = match ? match.handlers : this.fallbackHandlers;
      if (!handlers) return defaults;
      var cacheHandler = handlers.find(function (handler) {
        return handler.type === 'cache';
      });
      if (!cacheHandler || !cacheHandler.server || !cacheHandler.server.key) return defaults;
      return cacheHandler.server.key(request, defaults);
    }

    /**
     * Runs the current url (from env) and generates a result from each the matching route's handlers.
     * @param {Object} request The request being served
     * @param {String} request.path The url path
     * @param {String} request.method The http method
     * @param {Response} response The response object
     * @param {Object} options
     * @param {Boolean} [options.initialLoad=false] Set to true if this is the initial load of the application.  This will cause the HTML to be cached for the current path
     * @return {Object} Generates state objects
     */

  }, {
    key: 'run',
    value: function () {
      var _ref2 = _asyncGenerator3.default.wrap( /*#__PURE__*/_regenerator2.default.mark(function _callee2(request, response) {
        var _ref3 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
            _ref3$initialLoad = _ref3.initialLoad,
            initialLoad = _ref3$initialLoad === undefined ? false : _ref3$initialLoad,
            _ref3$historyState = _ref3.historyState,
            historyState = _ref3$historyState === undefined ? {} : _ref3$historyState;

        var _findMatchingRoute2, match, params, handlers, willFetchFromServer, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, handler, result;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _findMatchingRoute2 = this.findMatchingRoute(request), match = _findMatchingRoute2.match, params = _findMatchingRoute2.params;


                request.params = params;

                handlers = match ? match.handlers : this.fallbackHandlers;
                willFetchFromServer = !initialLoad && handlers.some(function (h) {
                  return h.type === 'fromServer';
                });

                // Here we ensure that the loading mask is displayed immediately if we are going to fetch from the server
                // and that the app state's location information is updated.

                if (!this.isBrowser) {
                  _context2.next = 9;
                  break;
                }

                _context2.next = 7;
                return (0, _extends3.default)({
                  loading: willFetchFromServer,
                  location: {
                    protocol: location.protocol.replace(/:/, ''),
                    pathname: location.pathname,
                    search: location.search,
                    hostname: location.hostname,
                    port: location.port
                  }
                }, historyState);

              case 7:
                _context2.next = 10;
                break;

              case 9:
                this.parseBody(request);

              case 10:
                _context2.prev = 10;
                _iteratorNormalCompletion3 = true;
                _didIteratorError3 = false;
                _iteratorError3 = undefined;
                _context2.prev = 14;
                _iterator3 = handlers[Symbol.iterator]();

              case 16:
                if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                  _context2.next = 36;
                  break;
                }

                handler = _step3.value;

                if (typeof handler === 'function') {
                  handler = {
                    runOn: { server: true, client: true },
                    fn: handler
                  };
                }

                // skip state handlers on initial hydration - we just need to run cache and track

                if (!(initialLoad && handler.type !== 'cache')) {
                  _context2.next = 21;
                  break;
                }

                return _context2.abrupt('continue', 33);

              case 21:
                if (!(!handler.runOn.client && this.isBrowser)) {
                  _context2.next = 23;
                  break;
                }

                return _context2.abrupt('continue', 33);

              case 23:
                if (!(!handler.runOn.server && !this.isBrowser)) {
                  _context2.next = 25;
                  break;
                }

                return _context2.abrupt('continue', 33);

              case 25:
                if (!(request.pathname.endsWith('.json') && (handler.runOn.server !== true || this.isBrowser))) {
                  _context2.next = 27;
                  break;
                }

                return _context2.abrupt('continue', 33);

              case 27:
                _context2.next = 29;
                return _asyncGenerator3.default.await(this.toPromise(handler.fn, params, request, response));

              case 29:
                result = _context2.sent;

                if (!result) {
                  _context2.next = 33;
                  break;
                }

                _context2.next = 33;
                return result;

              case 33:
                _iteratorNormalCompletion3 = true;
                _context2.next = 16;
                break;

              case 36:
                _context2.next = 42;
                break;

              case 38:
                _context2.prev = 38;
                _context2.t0 = _context2['catch'](14);
                _didIteratorError3 = true;
                _iteratorError3 = _context2.t0;

              case 42:
                _context2.prev = 42;
                _context2.prev = 43;

                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }

              case 45:
                _context2.prev = 45;

                if (!_didIteratorError3) {
                  _context2.next = 48;
                  break;
                }

                throw _iteratorError3;

              case 48:
                return _context2.finish(45);

              case 49:
                return _context2.finish(42);

              case 50:

                if (initialLoad && response.clientCache === 'force-cache') {
                  this.cacheInitialState(request, response);
                }
                _context2.next = 57;
                break;

              case 53:
                _context2.prev = 53;
                _context2.t1 = _context2['catch'](10);
                _context2.next = 57;
                return this.errorHandler(_context2.t1, params, request, response);

              case 57:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[10, 53], [14, 38, 42, 50], [43,, 45, 49]]);
      }));

      function run(_x4, _x5) {
        return _ref2.apply(this, arguments);
      }

      return run;
    }()

    /**
     * Runs all client and server handlers for the specified path and method
     * @param {Object} request The request being served
     * @param {String} request.path The url path
     * @param {String} request.method The http method
     * @param {Response} response The response object
     * @param {Object} options
     * @param {Object} [state={}] The accumulated state from other handlers
     * @return {Object} The merged result of all handlers
     */

  }, {
    key: 'runAll',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(request, response, options) {
        var state = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

        var _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, _value2, result;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                state = (0, _lodash3.cloneDeep)(state); // prevent initial state from being mutated

                _iteratorNormalCompletion4 = true;
                _didIteratorError4 = false;
                _iteratorError4 = undefined;
                _context3.prev = 4;
                _iterator4 = (0, _asyncIterator3.default)(this.run(request, response, options));

              case 6:
                _context3.next = 8;
                return _iterator4.next();

              case 8:
                _step4 = _context3.sent;
                _iteratorNormalCompletion4 = _step4.done;
                _context3.next = 12;
                return _step4.value;

              case 12:
                _value2 = _context3.sent;

                if (_iteratorNormalCompletion4) {
                  _context3.next = 19;
                  break;
                }

                result = _value2;

                if (typeof result === 'string') {
                  state = result;
                } else {
                  (0, _lodash3.merge)(state, result);
                }

              case 16:
                _iteratorNormalCompletion4 = true;
                _context3.next = 6;
                break;

              case 19:
                _context3.next = 25;
                break;

              case 21:
                _context3.prev = 21;
                _context3.t0 = _context3['catch'](4);
                _didIteratorError4 = true;
                _iteratorError4 = _context3.t0;

              case 25:
                _context3.prev = 25;
                _context3.prev = 26;

                if (!(!_iteratorNormalCompletion4 && _iterator4.return)) {
                  _context3.next = 30;
                  break;
                }

                _context3.next = 30;
                return _iterator4.return();

              case 30:
                _context3.prev = 30;

                if (!_didIteratorError4) {
                  _context3.next = 33;
                  break;
                }

                throw _iteratorError4;

              case 33:
                return _context3.finish(30);

              case 34:
                return _context3.finish(25);

              case 35:
                return _context3.abrupt('return', state);

              case 36:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[4, 21, 25, 35], [26,, 30, 34]]);
      }));

      function runAll(_x7, _x8, _x9) {
        return _ref4.apply(this, arguments);
      }

      return runAll;
    }()

    /**
     * Parses JSON and form body content
     * @private
     * @param {String} body The request body
     * @param {String} contentType The content-type header
     * @return {Object}
     */

  }, {
    key: 'parseBody',
    value: function parseBody(request) {
      if (!request.headers || !request.body) return;

      var contentType = (request.headers['content-type'] || '').toLowerCase();
      var body = request.body;


      if (contentType === 'application/json') {
        try {
          request.body = JSON.parse(body);
        } catch (e) {
          throw new Error('could not parse request body as application/json: ' + e.message);
        }
      } else if (contentType === 'application/x-www-form-urlencoded') {
        try {
          request.body = _qs2.default.parse(body);
        } catch (e) {
          throw new Error('could not parse request body as x-www-form-urlencoded: ' + e.message);
        }
      } else if (contentType.startsWith('multipart/form-data')) {
        try {
          request.body = (0, _parseMultipartRequest2.default)(request);
        } catch (e) {
          throw new Error('could not parse request body as multipart/form-data: ' + e.message);
        }
      }
    }

    /**
     * Converts specified callback to a promise
     * @param {Function/Object} callback A function that returns a Promise that
     *  resolves to the new state, a function that returns the new state, or the new state itself.
     * @param {Object} params The request parameters
     * @param {Object} request The request object with body and headers
     * @param {Response} response The response object
     */

  }, {
    key: 'toPromise',
    value: function toPromise(callback, params, request, response) {
      if ((0, _lodash2.default)(callback)) {
        var result = callback(params, request, response);

        if (result && result.then) {
          // callback returned a promise
          return result;
        } else {
          // callback returned the new state
          return Promise.resolve(result);
        }
      } else {
        // callback is the new state
        return Promise.resolve(callback);
      }
    }

    /**
     * Returns the matching route and parsed params for the specified path and method
     * @param {Object} request The http request
     * @return {Object} an object with match and params
     */

  }, {
    key: 'findMatchingRoute',
    value: function findMatchingRoute(request) {
      var params = void 0;
      var pathname = request.pathname,
          search = request.search,
          _request$method = request.method,
          method = _request$method === undefined ? 'GET' : _request$method;

      var path = pathname + search;

      method = method.toUpperCase();

      var match = this.routes.filter(function (route) {
        return method === route.method;
      }).find(function (route) {
        return params = route.path.match(path);
      });

      return { match: match, params: (0, _extends3.default)({}, params, _qs2.default.parse(search, { ignoreQueryPrefix: true })) };
    }

    /**
     * Returns true if the route will result in the server connecting to the
     * upstream site due to the presence of a `proxyUpstream` handler, otherwise
     * false.
     * @private
     * @param {Object} request
     * @return {Boolean}
     */

  }, {
    key: 'willFetchFromUpstream',
    value: function willFetchFromUpstream(request) {
      var _findMatchingRoute3 = this.findMatchingRoute(request),
          match = _findMatchingRoute3.match;

      var handlers = match ? match.handlers : this.fallbackHandlers;
      return handlers.some(function (handler) {
        return handler.type === 'proxyUpstream';
      });
    }

    /**
     * Called when the location is changed on the client
     * @param {Function} callback A callback to pass the new state to
     * @param {Object} location The new location
     */

  }, {
    key: 'reloadFromServer',


    /**
     * Call this function when client-side navigation to a non-PWA route has occurred.  This
     * will revert window.history to the previous url and reload the original destination URL
     * from the server.
     * @private
     */
    value: function reloadFromServer() {
      var href = window.location.href;

      this.history.goBack();
      self.location.href = href;
    }

    /**
     * Calls the specified callback whenever the current URL changes
     * @param {History} history
     * @param {Function} callback
     * @return {Router} this
     */

  }, {
    key: 'watch',
    value: function watch(history, callback) {
      this.history = history;
      this.prevLocation = history.location;

      history.listen(this.onLocationChange.bind(this, callback));

      var _history$location = history.location,
          pathname = _history$location.pathname,
          search = _history$location.search;

      var request = { pathname: pathname, search: search, method: 'GET' };
      var response = new _Response2.default(request);

      this.runAll(request, response, { initialLoad: true }, window.initialState);
      this.emit('after', { request: request, response: response, initialLoad: true });

      return this;
    }

    /**
     * Provides an easy way to navigate by changing some but not all of the query params.  Any keys
     * included in the params object are applied as new query param values.  All other query params are preserved.
     * @param {Object} params Key/value pairs to apply to the query string.  Specifying a value of undefined or null will remove that parameter from the query string
     */

  }, {
    key: 'applySearch',
    value: function applySearch(params) {
      var history = this.history;


      var nextParams = _qs2.default.stringify((0, _extends3.default)({}, _qs2.default.parse(history.location.search, { ignoreQueryPrefix: true }), params));

      history.push(history.location.pathname + '?' + nextParams);
    }

    /**
     * Gets the query string parameters for the current url as an object of key/value pairs
     * @return {Object}
     */

  }, {
    key: 'getQueryParams',
    value: function getQueryParams() {
      var history = this.history;

      return _qs2.default.parse(history.location.search, { ignoreQueryPrefix: true });
    }
  }]);
  return Router;
}(_eventemitter2.default);

exports.default = Router;
//# sourceMappingURL=Router.js.map