'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * Fetch's state as json from the specified url
 * @param {String} url The url to fetch
 * @param {Object} options 
 * @param {String} options.cache Set to "force-cache" to cache the response in the service worker.  Omit to skip the service worker cache.
 * @return {Object} A state patch
 */
var fetch = exports.fetch = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(url) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$cache = _ref2.cache,
        cache = _ref2$cache === undefined ? 'default' : _ref2$cache;

    var _location, href, apiVersion, result;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            (0, _serviceWorker.abortPrefetches)();
            doFetch = doFetch || (0, _fetchLatest.fetchLatest)(require('isomorphic-unfetch'));

            _location = location, href = _location.href;
            _context.prev = 3;
            apiVersion = (window.moov || {}).apiVersion;
            _context.next = 7;
            return doFetch(url, {
              cache: cache || "default",
              credentials: 'include',
              headers: {
                "x-react-storefront": "true", // allows back end handlers to quickly identify PWA API requests,
                "x-moov-api-version": apiVersion // needed for the service worker to determine the correct runtime cache name and ensure that we're not getting a cached response from a previous api version
              }
            }).then(function (response) {
              var redirected = response.redirected,
                  url = response.url;


              if (redirected) {
                redirectTo(url);
              } else {
                (0, _serviceWorker.resumePrefetches)();
                return response.json();
              }
            });

          case 7:
            result = _context.sent;

            if (!(result != null && location.href === href)) {
              _context.next = 10;
              break;
            }

            return _context.abrupt('return', (0, _extends3.default)({ loading: false }, result));

          case 10:
            _context.next = 19;
            break;

          case 12:
            _context.prev = 12;
            _context.t0 = _context['catch'](3);

            if (!_fetchLatest.StaleResponseError.is(_context.t0)) {
              _context.next = 18;
              break;
            }

            return _context.abrupt('return', null);

          case 18:
            throw _context.t0;

          case 19:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[3, 12]]);
  }));

  return function fetch(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Handles a redirect response.  Will do a client side navigation if the URL has the same hostname as the app, otherwise will
 * reload the page.
 * @param {String} url 
 */


exports.default = fromServer;

var _fetchLatest = require('../fetchLatest');

var _serviceWorker = require('./serviceWorker');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
var doFetch = void 0;function redirectTo(url) {
  if (url) {
    var parsed = new URL(url);
    var history = window.moov.history;


    if (parsed.hostname === window.location.hostname) {
      history.push(parsed.pathname + parsed.search);
    } else {
      window.location.assign(url);
    }
  } else {
    throw new Error('Received a redirect without a location header.');
  }
}

/**
 * Creates a handler that fetches data from the server.
 * 
 * The `handlerPath` should point to a module that exports a function that takes params, request, and response, 
 * and returns an object that should be applied to the app state.  For example:
 * 
 *  // routes.js
 *  router.get('/p/:id'
 *    fromServer('./product/product-handler')
 *  )
 * 
 *  // product/product-handler.js
 *  export default function productHandler(params, request, response) {
 *    return fetchFromUpstreamApi(`/products/${params.id}`)
 *      .then(res => res.json())
 *      .then(productData => ({ // the shape of this object should match your AppModel
 *        page: 'Product',
 *        product: productData
 *      }))
 *  }
 * 
 * When the request path ends in ".json", the json response will be returned verbatim.  In all other cases, server-side rendered HTML
 * will be returned.
 * 
 * You can also send a verbatim string response using `response.send(body)`.  For example:
 * 
 *  // routes.js
 *  router.get('/my-api'
 *    fromServer('./my-api-handler')
 *  )
 * 
 *  // my-api-handler.js
 *  export default function myApiHandler(params, request, response) {
 *    response
 *      .set('content-type', response.JSON)
 *      .send(JSON.stringify({ foo: 'bar' }))
 *  }
 * 
 * When `response.send()` is called in a handler, react-storefront will never perform server-side rendering.
 * 
 * @param {String} handlerPath The path to the module that exports a handler function that returns state to apply
 * @param {Function} getURL An optional function that returns the back end url to call when fetching.  You only need 
 *   to specify this if you want to override the default URL.
 * @return {Function}
 */
function fromServer(handlerPath, getURL) {
  return {
    type: 'fromServer',
    runOn: {
      server: true,
      client: true // fromServer handlers run on the client too - we make an ajax request to get the state from the server
    },
    fn: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(params, request, response) {
        var url;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(typeof handlerPath === 'string')) {
                  _context2.next = 6;
                  break;
                }

                url = location.pathname + '.json' + location.search;


                if (getURL) {
                  url = getURL(url);
                }

                // handler path has not been transpiled, fetch the data from the server and return the result.
                return _context2.abrupt('return', fetch(url, { cache: response.clientCache }));

              case 6:
                if (!(handlerPath == null)) {
                  _context2.next = 8;
                  break;
                }

                throw new Error('You must provide a path to a handler in fromServer().  Please check your routes.');

              case 8:
                return _context2.abrupt('return', handlerPath(params, request, response));

              case 9:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function fn(_x3, _x4, _x5) {
        return _ref3.apply(this, arguments);
      }

      return fn;
    }()
  };
}
//# sourceMappingURL=fromServer.js.map