'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeOldCaches = exports.configureCache = exports.resumePrefetches = exports.abortPrefetches = exports.prefetchJsonFor = exports.cache = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
/**
 * Cache content using the service worker.  If content is not supplied, the service worker will fetch
 * the content from the server
 * @param {String} path The URI path of the request
 * @param {String} cacheData The data to cache
 */
var cache = exports.cache = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(path, cacheData) {
    var apiVersion;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context.sent) {
              _context.next = 5;
              break;
            }

            apiVersion = window.moov.apiVersion;


            if (cacheData) {
              navigator.serviceWorker.controller.postMessage({ action: 'cache-state', path: path, apiVersion: apiVersion, cacheData: cacheData });
            } else {
              navigator.serviceWorker.controller.postMessage({ action: 'cache-path', path: path, apiVersion: apiVersion });
            }

          case 5:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function cache(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Prefetches and caches JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */


var prefetchJsonFor = exports.prefetchJsonFor = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(path, includeSSR) {
    var url, _url;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (path) {
              _context2.next = 2;
              break;
            }

            return _context2.abrupt('return');

          case 2:
            if (path.startsWith('http')) {
              url = new URL(path);

              cache('' + url.origin + url.pathname + '.json' + url.search);
            } else {
              _url = new URL('http://z.z' + path);

              cache(_url.pathname + '.json' + _url.search);
            }
            if (includeSSR) {
              cache(path);
            }

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function prefetchJsonFor(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Prefetches and caches SSR and JSON for the specified path
 * @param {String} path A URL path for a page (without .json)
 */


/**
 * Aborts all in progress prefetches.  Call this function to prevent prefetching from blocking
 * more important requests, like page navigation.
 */
var abortPrefetches = exports.abortPrefetches = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context3.sent) {
              _context3.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({ action: 'abort-prefetches' });

          case 4:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function abortPrefetches() {
    return _ref3.apply(this, arguments);
  };
}();

/**
 * Resume queued prefetch requests which were cancelled to allow for more important requests
 */


var resumePrefetches = exports.resumePrefetches = function () {
  var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context4.sent) {
              _context4.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({ action: 'resume-prefetches' });

          case 4:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function resumePrefetches() {
    return _ref4.apply(this, arguments);
  };
}();

/**
 * Configures runtime caching options
 * @param {Object} options 
 * @param {Object} options.cacheName The name of the runtime cache
 * @param {Object} options.maxEntries The max number of entries to store in the cache
 * @param {Object} options.maxAgeSeconds The TTL in seconds for entries
 */


var configureCache = exports.configureCache = function () {
  var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(options) {
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context5.sent) {
              _context5.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({ action: 'configure-runtime-caching', options: options });

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function configureCache(_x5) {
    return _ref5.apply(this, arguments);
  };
}();

/**
 * Resolves when the service worker has been installed
 */


var waitForServiceWorkerController = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (!(!navigator.serviceWorker || !navigator.serviceWorker.controller)) {
              _context6.next = 2;
              break;
            }

            return _context6.abrupt('return', false);

          case 2:
            return _context6.abrupt('return', new Promise(function (resolve) {
              navigator.serviceWorker.ready.then(function () {
                if (navigator.serviceWorker.controller) {
                  return resolve(true);
                }
                navigator.serviceWorker.addEventListener('controllerchange', function () {
                  return resolve(true);
                });
              });
            }));

          case 3:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function waitForServiceWorkerController() {
    return _ref6.apply(this, arguments);
  };
}();

/**
 * Removes runtime caches for old versions of the api.  This ensures that all responses
 * are appropriate for the current version of the UI.
 */


var removeOldCaches = exports.removeOldCaches = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return waitForServiceWorkerController();

          case 2:
            if (!_context7.sent) {
              _context7.next = 4;
              break;
            }

            navigator.serviceWorker.controller.postMessage({ action: 'remove-old-caches', apiVersion: window.moov.apiVersion });

          case 4:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function removeOldCaches() {
    return _ref7.apply(this, arguments);
  };
}();

exports.prefetch = prefetch;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function prefetch(path) {
  cache(path);
  prefetchJsonFor(path);
}
//# sourceMappingURL=serviceWorker.js.map