'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createGenerateClassName;

var _styles = require('@material-ui/core/styles');

/**
 * Creates a class name generator for JSS.  
 * Adapted from https://github.com/cssinjs/jss/blob/master/src/utils/createGenerateClassName.js
 * @param {Boolean} [amp=false] Set to true when rendering amp.  When true, the smaller production class names will be used so 
 *  you can tell for sure whether or not your stylesheet is small enough to meet amp's size limit.
 * @return {Function}
 */
function createGenerateClassName() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$amp = _ref.amp,
      amp = _ref$amp === undefined ? false : _ref$amp;

  var nextId = 0;

  if (amp || process.env.MOOV_ENV === 'production') {
    return function (_rule, sheet) {
      return '' + sheet.options.classNamePrefix + nextId++;
    };
  } else {
    return (0, _styles.createGenerateClassName)();
  }
} /**
   * @license
   * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
   */
//# sourceMappingURL=createGenerateClassName.js.map