'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.injectScriptInHead = injectScriptInHead;
exports.injectCodeInHead = injectCodeInHead;
exports.removeScriptInHead = removeScriptInHead;
exports.removeCodeInHead = removeCodeInHead;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
/**
 * Method for checking script tag in <head>
 * @param {String} src path to script
 */
function isScriptInjected(src) {
  if (typeof document === 'undefined') {
    return false;
  }
  var injectedScripts = [].concat((0, _toConsumableArray3.default)(document.querySelectorAll('script[src^="' + src + '"]')));
  return injectedScripts.length > 0;
}

/**
 * Method for checking script tag in <head>
 * @param {String} code script code
 */
function isCodeInjected(code) {
  if (typeof document === 'undefined') {
    return false;
  }
  var injectedCodes = [].concat((0, _toConsumableArray3.default)(document.head.getElementsByTagName('script'))).filter(function (el) {
    return el.innerHTML === code;
  });
  return injectedCodes.length > 0;
}

/**
 * Method for injecting script tag in <head>
 * @param {String} src path to script
 * @param {Object} [params] Optional params
 * @param {Object} [params.attrs={}] Optional attributes to add to element
 * @param {Boolean} [params.once=true] If true then script will be injected only once
 * @returns {Promise} Promise returns on script's onload event
 */
function injectScriptInHead(src) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$attrs = _ref.attrs,
      attrs = _ref$attrs === undefined ? {} : _ref$attrs,
      _ref$once = _ref.once,
      once = _ref$once === undefined ? true : _ref$once;

  if (once && isScriptInjected(src)) {
    return Promise.resolve();
  }
  if (typeof document === 'undefined') {
    return Promise.resolve();
  }
  var promiseResponse = void 0;
  var scriptHeadElement = document.createElement('script');
  scriptHeadElement.src = src;
  Object.keys(attrs).forEach(function (key) {
    scriptHeadElement.setAttribute(key, attrs[key]);
  });

  promiseResponse = new Promise(function (resolve, reject) {
    scriptHeadElement.onload = resolve;
    scriptHeadElement.onerror = reject;
  });

  document.head.appendChild(scriptHeadElement);

  return promiseResponse;
}

/**
 * Method for injecting target code in <head>
 * @param {String} code script code
 * @param {Object} [params] optional params
 * @param {Boolean} [params.once=false] If true then script will be injected only once
 */
function injectCodeInHead(code) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$once = _ref2.once,
      once = _ref2$once === undefined ? true : _ref2$once;

  if (once && isCodeInjected(code)) {
    return;
  }
  if (typeof document === 'undefined') {
    return;
  }
  var scriptHeadElement = document.createElement('script');
  scriptHeadElement.appendChild(document.createTextNode(code));

  document.head.appendChild(scriptHeadElement);
}

/**
 * Method for removing script in <head>
 * @param {String} src path to script
*/
function removeScriptInHead(src) {
  [].concat((0, _toConsumableArray3.default)(document.querySelectorAll('script[src^="' + src + '"]'))).forEach(function (el) {
    return el.parentNode.removeChild(el);
  });
}

/**
 * Method for removing code  in <head>
 * @param {String} code script code
*/
function removeCodeInHead(code) {
  [].concat((0, _toConsumableArray3.default)(document.head.getElementsByTagName('script'))).filter(function (el) {
    return el.innerHTML === code;
  }).forEach(function (el) {
    return el.parentNode.removeChild(el);
  });
}

var scriptInjector = {
  injectScriptInHead: injectScriptInHead,
  injectCodeInHead: injectCodeInHead,
  removeScriptInHead: removeScriptInHead,
  removeCodeInHead: removeCodeInHead
};
exports.default = scriptInjector;
//# sourceMappingURL=scriptInjector.js.map