'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = withStyleProps;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _withStyles = require('@material-ui/core/styles/withStyles');

var _withStyles2 = _interopRequireDefault(_withStyles);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Decorator for mapping a component's props to JSS styles
 * @param {Object} styles
 * @return {React.Component}
 * 
 * Example:
 * 
 *  import withStyleProps from 'react-storefront'
 *  import React, { Component } from 'react'
 * 
 *  @withStyleProps(({ theme, height }) => ({
 *    root: {
 *      height: `${height}px`,
 *    }
 *  }))
 *  export default class MyComponent extends Component {
 *    render() {
 *      return <div className={this.props.classes.root}>{this.props.children}</div>
 *    }
 *  }
 */
/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */
function withStyleProps(styles, options) {
  return function (Component) {
    return function (props) {
      var Comp = (0, _withStyles2.default)(function (theme) {
        return styles((0, _extends3.default)({}, props, { theme: theme }));
      }, options)(Component);
      return _react2.default.createElement(Comp, props);
    };
  };
}
//# sourceMappingURL=withStyleProps.js.map